﻿@hierarchies @adoSpecific
Feature: Test Case hierarchies in Azure DevOps

This feature file contain generic hierarchy-related scenarios that are not dependent on a specific hierarchy type.

The "folder" hierarchy type is used in these scenarios as an example, but they are applicable to all. 

Rule: Should be able to add Test Cases to the Test Suites

@tc:1421
Scenario: Test Cases are added to Test Suites
	Given there is an Azure DevOps project with a Test Plan
	And there is an empty Test Suite 'Custom Suite 1' in the project
	And there is an empty Test Suite 'Custom Suite 2' in the project
	And there are feature files with scenarios in the local repository as
		| feature file                    | scenarios |
		| Custom Suite 1/Feature1.feature | S1        |
		| Custom Suite 2/Feature2.feature | S2        |
	And the synchronizer is configured as
		| setting             | value   |
		| hierarchies[0]/type | folders |
	When the local repository is synchronized with push
	Then the Test Suites should contain the Test Cases as
		| test suite path | test cases   |
		| Custom Suite 1  | Scenario: S1 |
		| Custom Suite 2  | Scenario: S2 |

Rule: Should attempt to create the Test Suite if does not exist

@tc:1422
Scenario: The configuration requires new Test Suites to be created
	Given there is an Azure DevOps project with a Test Plan
	And there are feature files with scenarios in the local repository as
		| feature file                    | scenarios |
		| Custom Suite 1/Feature1.feature | S1        |
	And the synchronizer is configured as
		| setting             | value   |
		| hierarchies[0]/type | folders |
	When the local repository is synchronized with push
	Then the Test Suite 'Custom Suite 1' should be created and contain the following Test Cases
		| test case    |
		| Scenario: S1 |

Rule: The removed scenarios (if detected) should be removed from the suite

@tc:1423
Scenario: The Test Case of a deleted scenario is removed from the Test Suite if a remote scope was configured
	Given there is an Azure DevOps project with an empty Test Suite 'Custom Suite 1'
	And a remote scope is configured
	And the synchronizer is configured as
		| setting             | value   |
		| hierarchies[0]/type | folders |
	And there is an existing synchronized feature file 'Custom Suite 1/Feature1.feature'
		"""
		Feature: My Feature
		
		@tc:[id-of-test-case-1]
		Scenario: S1
			When I do something
		
		@tc:[id-of-test-case-2]
		Scenario: S2
			When I do something
		"""
	When the feature file 'Custom Suite 1/Feature1.feature' is updated to:
		"""
		Feature: My Feature
		
		# S2 deleted
		
		@tc:[id-of-test-case-2]
		Scenario: S2
			When I do something
		"""
	And the local repository is synchronized with push
	Then the Test Suites should be created and contain the Test Cases as
		| test suite path | test cases   | description                |
		| Custom Suite 1  | Scenario: S2 | doesn't contain S1 anymore |

@bypass-integration
Rule: A hierarchy root can be specified

A hint is displayed when no root is specified

@tc:1424
Scenario: A Test Plan is used as hierarchy root
	Given there is an Azure DevOps project with a Test Plan
	And there are feature files with scenarios in the local repository as
		| feature file             | scenarios |
		| FolderA/Feature1.feature | S1        |
	And the synchronizer is configured as
		| setting                      | value              |
		| hierarchies[0]/type          | folders            |
		| hierarchies[0]/root/testPlan | #[id-of-test-plan] |
	When the local repository is synchronized with push
	Then the Test Suite 'FolderA' should be created in Test Plan '#[id-of-test-plan]'
	And the Test Suite 'FolderA' should contain the following Test Cases
		| test case    |
		| Scenario: S1 |

@tc:1425
Scenario: A Test Suite is used as hierarchy root
	All usual Test Suite specifyer (name, id, path, testPlan) can be used in 'root' setting

	Given there is an Azure DevOps project with an empty Test Suite 'Custom Hierarchy Root'
	And there are feature files with scenarios in the local repository as
		| feature file                     | scenarios |
		| FolderA/FolderB/Feature1.feature | S1        |
	And the synchronizer is configured as
		| setting                      | value                 |
		| hierarchies[0]/type          | folders               |
		| hierarchies[0]/root/testPlan | #[id-of-test-plan]    |
		| hierarchies[0]/root/path     | Custom Hierarchy Root |
	When the local repository is synchronized with push
	Then the Test Suite 'Custom Hierarchy Root/FolderA/FolderB' should be created in Test Plan '#[id-of-test-plan]'
	And the Test Suite 'FolderB' should contain the following Test Cases
		| test case    |
		| Scenario: S1 |

@bypass-integration
Rule: The Test Cases to be included to the hierarchy can be restricted by a condition

@tc:1426
Scenario: Only a subset of the scenarios are added to the hierarchy
	Given there is an Azure DevOps project
	And there are feature files with scenarios in the local repository as
		| feature file             | scenarios |
		| FolderA/Feature1.feature | S1        |
		| FolderB/Feature2.feature | S2        |
		| FolderB/Feature3.feature | S3        |
	And the synchronizer is configured as
		| setting                  | value                                   |
		| hierarchies[0]/type      | folders                                 |
		| hierarchies[0]/condition | not ($sourceFile ~ **/Feature2.feature) |
	When the local repository is synchronized with push
	Then the Test Suites should be created and contain the Test Cases as
		| test suite path | test cases   | description                              |
		| FolderA         | Scenario: S1 | doesn't contain S2 as it is feltered out |
		| FolderB         | Scenario: S3 |                                          |

@bypass-integration
Rule: Show warning by default when hierarchy contains additional Test Suites (can be suppressed)

@tc:1427
Scenario: The hierarchy contains an additional non-synchronized Test Suite
	Given there is an Azure DevOps project with an empty Test Suite 'Custom Hierarchy Root'
	And there is an empty Test Suite 'Custom Hierarchy Root/Other Suite' in the project
	And there are feature files with scenarios in the local repository as
		| feature file     | scenarios |
		| Feature1.feature | S1        |
	And the synchronizer is configured as
		| setting                  | value                 |
		| hierarchies[0]/type      | folders               |
		| hierarchies[0]/root/path | Custom Hierarchy Root |
	When SpecSync push is executed
	Then the Test Suite 'Custom Hierarchy Root' should contain the following Test Cases
		| test case    |
		| Scenario: S1 |
	But the command should finish with warnings
	And the output should contain "contains additional Test Suite(s) that are not part of the hierarchy: Other Suite"

@tc:1428
Scenario: The warning for additional Test Suites can be suppressed
	Given there is an Azure DevOps project with an empty Test Suite 'Custom Hierarchy Root'
	And there is an empty Test Suite 'Custom Hierarchy Root/Other Suite' in the project
	And there are feature files with scenarios in the local repository as
		| feature file     | scenarios |
		| Feature1.feature | S1        |
	And the synchronizer is configured as
		| setting                              | value                 |
		| hierarchies[0]/type                  | folders               |
		| hierarchies[0]/root/path             | Custom Hierarchy Root |
		| hierarchies[0]/ignoreAdditionalNodes | true                  |
	When SpecSync push is executed
	Then the command should succeed without warnings

@bypass-integration
Rule: Scenarios that are moved to another hierarchy node should be removed from the original one

@tc:1429
Scenario: The scenario is moved to another Test Suite
	Given there is an Azure DevOps project
	And the synchronizer is configured as
		| setting                           | value          |
		| hierarchies[0]/type               | custom         |
		| hierarchies[0]/nodes[0]/name      | Custom Suite 1 |
		| hierarchies[0]/nodes[0]/condition | @suite1        |
		| hierarchies[0]/nodes[1]/name      | Custom Suite 2 |
		| hierarchies[0]/nodes[1]/condition | @suite2        |
	And there is an existing synchronized feature file
		"""
		Feature: My Feature
		
		@suite1
		@tc:[id-of-test-case-1]
		Scenario: S1
			When I do something
		
		@suite1
		@tc:[id-of-test-case-2]
		Scenario: S2
			When I do something
		
		@suite2
		@tc:[id-of-test-case-3]
		Scenario: S3
			When I do something
		"""
	When the feature file is updated to:
		"""
		Feature: My Feature
		
		@suite1
		@tc:[id-of-test-case-1]
		Scenario: S1
			When I do something
		
		# tag of S2 is changed from @suite1 to @suite2
		@suite2
		@tc:[id-of-test-case-2]
		Scenario: S2
			When I do something
		
		@suite2
		@tc:[id-of-test-case-3]
		Scenario: S3
			When I do something
		"""
	And the local repository is synchronized with push
	Then the Test Suites should be created and contain the Test Cases as
		| test suite path | test cases                 | description                |
		| Custom Suite 1  | Scenario: S1               | doesn't contain S2 anymore |
		| Custom Suite 2  | Scenario: S2, Scenario: S3 |                            |

@enterprise-feature @bypass-integration
Rule: Multiple hierarchies can be specified

@tc:1430
Scenario: Test Cases are synchronized to multiple hierarchies
	Given there is an Azure DevOps project
	And there is an empty Test Suite 'Custom Root 1' in the project
	And there is an empty Test Suite 'Custom Root 2' in the project
	And there are feature files with scenarios in the local repository as
		| feature file             | scenarios |
		| FolderA/Feature1.feature | S1        |
	And the synchronizer is configured as
		| setting                  | value         |
		| hierarchies[0]/name      | H1            |
		| hierarchies[0]/type      | folders       |
		| hierarchies[0]/root/path | Custom Root 1 |
		| hierarchies[1]/name      | H2            |
		| hierarchies[1]/type      | folders       |
		| hierarchies[1]/root/path | Custom Root 2 |
	When the local repository is synchronized with push
	Then the Test Suites should contain the Test Cases as
		| test suite path       | test cases   |
		| Custom Root 1/FolderA | Scenario: S1 |
		| Custom Root 2/FolderA | Scenario: S1 |

@bypass-integration
Rule: Can publish test results to a hierarchy

@tc:1431
Scenario: Results published to a hierarchy
	The condition setting is also considered when specified
	The name 'default' does not have to be specified as that is the default name.

	Given there is an Azure DevOps project with a Test Plan
	And there are feature files with scenarios in the local repository as
		| feature file             | feature name   | scenarios |
		| FolderA/Feature1.feature | Sample feature | S1, S2    |
	And the synchronizer is configured as
		| setting                  | value            |
		| hierarchies[0]/name      | default          |
		| hierarchies[0]/type      | folders          |
		| hierarchies[0]/condition | not ($name = S2) |
	And the local repository has been synchronized
	And there is a test result file with
		| name |
		| S1   |
		| S2   |
	And the synchronizer is configured as
		| setting                           | value   |
		| publishTestResults/hierarchies[0] | default |
	When the test result is published successfully
	Then the there should be a Test Run registered with test results
		| test case    | test suite |
		| Scenario: S1 | FolderA    |

@tc:1432
@enterprise-feature
Scenario: Results published to multiple hierarchies
	Given there is an Azure DevOps project
	And there is an empty Test Suite 'Custom Root 1' in the project
	And there is an empty Test Suite 'Custom Root 2' in the project
	And there are feature files with scenarios in the local repository as
		| feature file             | feature name   | scenarios |
		| FolderA/Feature1.feature | Sample feature | S1        |
	And the synchronizer is configured as
		| setting                  | value         |
		| hierarchies[0]/name      | H1            |
		| hierarchies[0]/type      | folders       |
		| hierarchies[0]/root/path | Custom Root 1 |
		| hierarchies[1]/name      | H2            |
		| hierarchies[1]/type      | folders       |
		| hierarchies[1]/root/path | Custom Root 2 |
	And the local repository has been synchronized
	And there is a test result file with
		| name |
		| S1   |
	And the synchronizer is configured as
		| setting                           | value |
		| publishTestResults/hierarchies[0] | H1    |
		| publishTestResults/hierarchies[1] | H2    |
	When the test result is published successfully
	Then the there should be a Test Run registered with test results
		| test case    | test suite path       |
		| Scenario: S1 | Custom Root 1/FolderA |
		| Scenario: S1 | Custom Root 2/FolderA |